# -*- coding: utf-8 -*-
# (c) YashDK [yash-dk@github]

import asyncio, aria2p, logging, os
from ..core.getVars import get_val
from telethon.tl.types import KeyboardButtonCallback
from telethon.errors.rpcerrorlist import MessageNotModifiedError

# referenced from public leech
# pylint: disable=no-value-for-parameter
torlog = logging.getLogger(__name__)

async def aria_start():
    aria2_daemon_start_cmd = []
    # start the daemon, aria2c command
    aria2_daemon_start_cmd.append("aria2c")
    # aria2_daemon_start_cmd.append("--allow-overwrite=true")
    aria2_daemon_start_cmd.append("--daemon=true")
    aria2_daemon_start_cmd.append("--enable-rpc")
    aria2_daemon_start_cmd.append("--follow-torrent=false")
    aria2_daemon_start_cmd.append("--max-connection-per-server=8")
    aria2_daemon_start_cmd.append("--min-split-size=10M")
    aria2_daemon_start_cmd.append("--rpc-listen-all=false")
    aria2_daemon_start_cmd.append(f"--rpc-listen-port=8100")
    aria2_daemon_start_cmd.append("--rpc-max-request-size=1024M")
    aria2_daemon_start_cmd.append("--seed-ratio=0.0")
    aria2_daemon_start_cmd.append("--seed-time=1")
    aria2_daemon_start_cmd.append("--split=10")
    aria2_daemon_start_cmd.append(f"--bt-stop-timeout=100")
    #
    torlog.debug(aria2_daemon_start_cmd)
    #
    process = await asyncio.create_subprocess_exec(
        *aria2_daemon_start_cmd,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE
    )
    stdout, stderr = await process.communicate()
    torlog.debug(stdout)
    torlog.debug(stderr)
    aria2 = aria2p.API(
        aria2p.Client(
            host="http://localhost",
            port=8100,
            secret=""
        )
    )
    return aria2

def add_magnet(aria_instance, magnetic_link, c_file_name):
    try:
        download = aria_instance.add_magnet(
            magnetic_link
        )
    except Exception as e:
        return False, "⚠ **FAILED** \n" + str(e) + " \nPlease do not send SLOW links."
    else:
        return True, "" + download.gid + ""


def add_torrent(aria_instance, torrent_file_path):
    if torrent_file_path is None:
        return False, "⚠ **FAILED** \n\nSomething went wrong when trying to add <u>TORRENT</u> file!"
    if os.path.exists(torrent_file_path):
        # Add Torrent Into Queue
        try:
            download = aria_instance.add_torrent(
                torrent_file_path,
                uris=None,
                options=None,
                position=None
            )
        except Exception as e:
            return False, "⚠ **FAILED** \n" + str(e) + " \nPlease do not send SLOW links."
        else:
            return True, "" + download.gid + ""
    else:
        return False, "⚠ **FAILED** \n" + str(e) + " \nPlease try other sources to get workable link."


def add_url(aria_instance, text_url, c_file_name):
    uris = [text_url]
    # Add URL Into Queue
    try:
        download = aria_instance.add_uris(
            uris
        )
    except Exception as e:
        return False, "⚠ **FAILED** \n" + str(e) + " \nPlease do not send SLOW links."
    else:
        return True, "" + download.gid + ""

async def check_metadata(aria2, gid):
    file = aria2.get_download(gid)
    torlog.info(file)
    if not file.followed_by_ids:
        return None
    new_gid = file.followed_by_ids[0]
    torlog.info("Changing GID " + gid + " to " + new_gid)
    return new_gid

async def aria_dl(
    incoming_link,
    c_file_name,
    sent_message_to_update_tg_p,
    user_msg
):
    aria_instance = await aria_start()
    if incoming_link.lower().startswith("magnet:"):
        sagtus, err_message = add_magnet(aria_instance, incoming_link, c_file_name)
    elif incoming_link.lower().endswith(".torrent"):
        #sagtus, err_message = add_torrent(aria_instance, incoming_link)
        #sagtus, err_message = add_url(aria_instance, incoming_link, c_file_name)
        return False, "⚠ Cannot download this .torrent file! "
    else:
        sagtus, err_message = add_url(aria_instance, incoming_link, c_file_name)
    if not sagtus:
        return sagtus, err_message
    torlog.info(err_message)

    op = await check_progress_for_dl(
        aria_instance,
        err_message,
        sent_message_to_update_tg_p,
        None,
        user_msg=user_msg
    )
    if incoming_link.startswith("magnet:"):
        #
        err_message = await check_metadata(aria_instance, err_message)
        #
        await asyncio.sleep(1)
        if err_message is not None:
            op = await check_progress_for_dl(
                aria_instance,
                err_message,
                sent_message_to_update_tg_p,
                None,
                user_msg=user_msg
            )
        else:
            return False, "can't get metadata \n\n#stopped"
    await asyncio.sleep(1)
    
    if op is None:
        return False, "⚠ Known error. Nothing wrong here. You didnt follow instructions."
    else:
        statusr, stmsg = op
        if statusr:
            file = aria_instance.get_download(err_message)
            to_upload_file = file.name
        
            return True, to_upload_file
        else:
            return False, stmsg

async def check_progress_for_dl(aria2, gid, event, previous_message, rdepth = 0, user_msg=None):
    try:
        file = aria2.get_download(gid)
        complete = file.is_complete
        if not complete:
            if not file.error_message:
                msg = ""
                downloading_dir_name = "getting data... "
                try:
                    downloading_dir_name = str(file.name)
                except:
                    pass

                msg = f"\n<b>downloading:</b> <code>{downloading_dir_name}</code>"
                msg += f"\n<b>speed:</b> <code>{file.download_speed_string()}</code>"
                msg += f"\n<b>progress:</b> <code>{file.progress_string()}</code>"
                msg += f"\n<b>eta:</b> <code>{file.eta_string()}</code>"
                msg += f"\n<b>size:</b> <code>{file.total_length_string()}</code>"
                msg += f"\n\n  <i>using engine:</i> <code>aria2</code>"
                if file.seeder is False:
                    """https://t.me/c/1220993104/670177"""
                    #msg += f"| S: {file.num_seeders} |"
                # msg += f"\nStatus: {file.status}"
                #msg += f"\n<code>/cancel {gid}</code>"
                
                # format :- torcancel <provider> <identifier>
                if user_msg is None:
                    mes = await event.get_reply_message()
                else:
                    mes = user_msg
                data = f"torcancel aria2 {gid} {mes.sender_id}"
                
                # LOGGER.info(msg)
                if msg != previous_message:
                    if rdepth < 3:
                        await event.edit(msg,parse_mode="html", buttons=[KeyboardButtonCallback("Cancel Direct Leech",data=data.encode("UTF-8"))])
                    else:
                        await event.edit(msg,parse_mode="html")
                    previous_message = msg
            else:
                msg = file.error_message
                await event.edit(f"`{msg}`",parse_mode="html", buttons=None)
                torlog.error(f"⚠ The aria2 download faild due to this reason:- {msg}")
                return False, f"⚠ The aria2 download faild due to this reason:- {msg}"
            await asyncio.sleep(get_val("EDIT_SLEEP_SECS"))
            
            # TODO idk not intrested in using recursion here
            return await check_progress_for_dl(
                aria2, gid, event, previous_message,user_msg=mes
            )
        else:
            await event.edit(f"<b>Download completed:</b> <code>{file.name}</code>\n Trying to upload... ",parse_mode="html", buttons=None)
            return True, "Download Complete"
    except aria2p.client.ClientException as e:
        if " not found" in str(e) or "'file'" in str(e):
            fname = "getting data... "
            try:
                fname = file.name
            except:pass

            return False, f"The Download was canceled. {fname}"
        else:
            torlog.warning("Errored due to ta client error.")
        pass
    except MessageNotModifiedError:
        pass
    except RecursionError:
        file.remove(force=True)
        return False, "⚠ The link is basically dead."
    except Exception as e:
        torlog.info(str(e))
        if " not found" in str(e) or "'file'" in str(e):
            return False, "The Download was canceled."
        else:
            torlog.warning(str(e))
            return False, f"Error: {str(e)}"

async def remove_dl(gid):
    aria2 = await aria_start()
    try:
        downloads = aria2.get_download(gid)
        downloads.remove(force=True, files=True)
    except:
        torlog.exception("exc")
        pass



# -*- coding: utf-8 -*-

import re,os,shutil,time, aiohttp
from telethon.tl import types
import logging, os, shutil
import asyncio as aio
import requests
import urllib.parse
from bs4 import BeautifulSoup
from . import QBittorrentWrap
from . import ariatools
from .tele_upload import upload_handel
from .rclone_upload import rclone_driver
from .zip7_utils import add_to_zip, extract_archive
from ..core.getVars import get_val

logging.basicConfig(level=logging.DEBUG)
logging.getLogger("telethon").setLevel(logging.WARNING)
torlog = logging.getLogger(__name__)

#this files main task is to keep the ability to switch to a new engine if needed ;)

#TODO implement multiple magnets from same message if needed
#this function is to ensure that only one magnet is passed at a time
def get_magnets(text):
    matches = [ i for i in re.finditer("magnet:",text)]
    magnets = list()

    for i in range(len(matches)):
        if i == len(matches)-1:
            magnets.append(text[matches[i].span()[0]:])
        elif i == 0:
            magnets.append(text[:matches[i+1].span()[0]])
        else:
            magnets.append(text[matches[i].span()[0]:matches[i+1].span()[0]])

    for i in range(len(magnets)):
        magnets[i] = magnets[i].strip()

    return magnets[0]


def get_entities(msg):
    urls = list()

    for i in msg.entities:
        if isinstance(i,types.MessageEntityUrl):
            o,l = i.offset,i.length
            urls.append(msg.text[o:o+l])
        elif isinstance(i,types.MessageEntityTextUrl):
            urls.append(i.url)

    if len(urls) > 0:
        return urls[0]
    else:
        return None
    
   
    
async def check_link(msg,rclone=False,is_zip=False, extract=False):
    urls = None
    print("here2")
    omess = msg
    msg = await msg.get_reply_message()

    if extract:
        mess = f"You chose to extract the archive <a href='tg://user?id={omess.sender_id}'>ENTER PASSWORD IF ANY.</a>\n Use <code>/setpass {omess.id} password-here</code>"
        omess.client.dl_passwords[omess.id] = [str(omess.sender_id), None]
        await omess.reply(mess, parse_mode="html")

    if msg is None:
        urls = None

    elif msg.document is not None:
        name = None
        for i in msg.document.attributes:
            if isinstance(i,types.DocumentAttributeFilename):
                name = i.file_name
        
        if name is None:
            await omess.reply("This is not a torrent file to leech from. Send <code>.torrent</code> file",parse_mode="html")
        elif name.lower().endswith(".torrent"):
            rmess = await omess.reply("Downloading the torrent file.")

            #not worring about the download location now
            # TODO do something to de register the torrents
            path = await msg.download_media()
            rval =  await QBittorrentWrap.register_torrent(path,rmess,omess,file=True)
            
            if not isinstance(rval,bool) and rval is not None:
                if extract:
                    newpath = await handle_ext_zip(rval[0], rmess, omess)
                    if not newpath is False:
                        rval[0] = newpath
                else:
                    newpath = await handle_zips(rval[0], is_zip, rmess)
                    if newpath is False:
                        pass
                    else:
                        rval[0] = newpath

                if not rclone:
                    rdict = await upload_handel(rval[0],rmess,omess.from_id,dict(),user_msg=omess)
                    await print_files(omess,rdict,rval[1])
                    torlog.info("Here are the files uploaded {}".format(rdict))
                    await QBittorrentWrap.delete_this(rval[1])
                else:
                    res = await rclone_driver(rval[0],rmess,omess)
                    if res is None:
                        await msg.reply("<b>UPLOAD TO DRIVE FAILED CHECK LOGS</b>",parse_mode="html")
                    await QBittorrentWrap.delete_this(rval[1])

            try:
                
                os.remove(path)
                if os.path.isdir(rval):
                    shutil.rmtree(rval)
                else:
                    os.remove(rval)
            except:pass
            return rval
        else:
            await omess.reply("This is not a torrent file to leech from. Send <code>.torrent</code> file",parse_mode="html")

    elif msg.raw_text is not None:
        if msg.raw_text.lower().startswith("magnet:"):
            rmess = await omess.reply("Scanning....")
            
            mgt = get_magnets(msg.raw_text.strip())
            path = await QBittorrentWrap.register_torrent(mgt,rmess,omess,True)
            
            if not isinstance(path,bool) and path is not None:
                if extract:
                    newpath = await handle_ext_zip(path[0], rmess, omess)
                    if not newpath is False:
                        path[0] = newpath
                else:
                    newpath = await handle_zips(path[0], is_zip, rmess)
                    if newpath is False:
                        pass
                    else:
                        path[0] = newpath


                if not rclone:
                    rdict = await upload_handel(path[0],rmess,omess.from_id,dict(),user_msg=omess)
                    await print_files(omess,rdict,path[1])
                    torlog.info("Here are the files to be uploaded {}".format(rdict))
                    await QBittorrentWrap.delete_this(path[1])
                else:
                    res = await rclone_driver(path[0],rmess,omess)
                    if res is None:
                        await msg.reply("<b>UPLOAD TO DRIVE FAILED CHECK LOGS</b>",parse_mode="html")
                    await QBittorrentWrap.delete_this(path[1])
            try:
                
                if os.path.isdir(path):
                    shutil.rmtree(path)
                else:
                    os.remove(path)
            except:pass
        elif msg.raw_text.lower().endswith(".torrent"):
            rmess = await omess.reply("Downloading the torrent file.")

            #not worring about the download location now
            # TODO do something to de register the torrents
            path = ""
            async with aiohttp.ClientSession() as sess:
                async with sess.get(msg.raw_text) as resp:
                    if resp.status == 200:
                        path = str(time.time()).replace(".","")+".torrent"
                        with open(path, "wb") as fi:
                            fi.write(await resp.read())
                    else:
                        await rmess.edit("Error got HTTP response code:- "+str(resp.status))
                        return    

            rval =  await QBittorrentWrap.register_torrent(path,rmess,omess,file=True)
            
            if not isinstance(rval,bool) and rval is not None:
                if extract:
                    newpath = await handle_ext_zip(rval[0], rmess, omess)
                    if not newpath is False:
                        rval[0] = newpath
                else:
                    newpath = await handle_zips(rval[0], is_zip, rmess)
                    if newpath is False:
                        pass
                    else:
                        rval[0] = newpath

                if not rclone:
                    rdict = await upload_handel(rval[0],rmess,omess.from_id,dict(),user_msg=omess)
                    await print_files(omess,rdict,rval[1])
                    torlog.info("Here are the files uploaded {}".format(rdict))
                    await QBittorrentWrap.delete_this(rval[1])
                else:
                    res = await rclone_driver(rval[0],rmess,omess)
                    if res is None:
                        await msg.reply("<b>UPLOAD TO DRIVE FAILED CHECK LOGS</b>",parse_mode="html")
                    await QBittorrentWrap.delete_this(rval[1])

            try:
                
                os.remove(path)
                if os.path.isdir(rval):
                    shutil.rmtree(rval)
                else:
                    os.remove(rval)
            except:pass
            return rval

        elif msg.entities is not None:
            urls = get_entities(msg)
            # currently discontinuing the depending on the entities as its eratic
            url = None
            rmsg = await omess.reply("Scanning....")
            await aio.sleep(1)
            if 'zippyshare.com' in urls:
                await rmsg.edit("Zippyshare Link Found")
                await aio.sleep(2)
                try:
                    link = re.findall(r'\bhttps?://.*zippyshare\.com\S+', urls)[0]
                except:
                    await rmsg.edit("No ZippyShare links found")
                session = requests.Session()
                base_url = re.search('http.+.com', link).group()
                response = session.get(link)
                page_soup = BeautifulSoup(response.content, "lxml")
                scripts = page_soup.find_all("script", {"type": "text/javascript"})
                for script in scripts:
                    if "getElementById('dlbutton')" in script.text:
                        url_raw = re.search(r'= (?P<url>\".+\" \+ (?P<math>\(.+\)) .+);',
                                            script.text).group('url')
                        math = re.search(r'= (?P<url>\".+\" \+ (?P<math>\(.+\)) .+);',
                           script.text).group('math')
                        url = url_raw.replace(math, '"' + str(eval(math)) + '"')
                        break
                url = base_url + eval(url)
                name = urllib.parse.unquote(url.split('/')[-1])
                #torlog.info(dl_url)
 #               return url
            
            elif 'mediafire.com' in urls:
                await rmsg.edit("Mediafire Link Found")
                await aio.sleep(2)
                try:
                    link = re.findall(r'\bhttps?://.*mediafire\.com\S+', urls)[0]
                except:
                    await omess.reply("No ZippyShare links found")
                page = BeautifulSoup(requests.get(link).content, 'lxml')
                info = page.find('a', {'aria-label': 'Download file'})
                url = info.get('href')
                #torlog.info(dl_url)
#                return url
                
            else:
#                return url
                url = None
                await rmsg.edit("Processing Link")
                await aio.sleep(1)
            #todo implement direct links ;)
             
            # weird stuff had to refect message
#            url = None
#            umsg = await rmess.edit("processsing link)
            
            torlog.info(msg.raw_text)
            torlog.info("Downloading Urls")
            torlog.info(url)
            torlog.info(urls)
            rmsg = await omess.client.get_messages(ids=rmsg.id, entity=rmsg.chat_id)
            try:
                if url is None:
                    stat, path = await ariatools.aria_dl(msg.raw_text,"",rmsg,omess)
                elif url is not None:
                    stat, path = await ariatools.aria_dl(url,"",rmsg,omess)                
                else:
                    stat, path = await ariatools.aria_dl(urls,"",rmsg,omess)
            except:pass
                                
            if not isinstance(path,bool) and stat:
                if extract:
                    newpath = await handle_ext_zip(path, rmsg, omess)
                    if not newpath is False:
                        path = newpath
                else:
                    newpath = await handle_zips(path, is_zip, rmsg)
                    if newpath is False:
                        pass
                    else:
                        path = newpath
                
                if not rclone:
                    rdict = await upload_handel(path,rmsg,omess.from_id,dict(),user_msg=omess)
                    await print_files(omess,rdict)
                    torlog.info("Here are the files to be uploaded {}".format(rdict))
                else:
                    res = await rclone_driver(path,rmsg, omess)
                    if res is None:
                        await msg.reply("<b>UPLOAD TO DRIVE FAILED CHECK LOGS</b>",parse_mode="html")
            elif stat is False:
                await rmsg.edit("Failed to download this file.\n"+str(path))
            
            try:
                if os.path.isdir(path):
                    shutil.rmtree(path)
                else:
                    os.remove(path)
            except:pass
        else:
            torlog.info("Downloading Url")
            #consider it as a direct link LOL
            rmsg = await omess.reply("processing")

            stat, path = await ariatools.aria_dl(omess.raw_text,"",rmsg,omess)
            if not isinstance(path,bool) and stat:
                if extract:
                    newpath = await handle_ext_zip(path, rmsg, omess)
                    if not newpath is False:
                        path = newpath
                else:
                    newpath = await handle_zips(path, is_zip, rmsg)
                    if newpath is False:
                        pass
                    else:
                        path = newpath
                
                if not rclone:
                    rdict = await upload_handel(path,rmsg,omess.from_id,dict(),user_msg=omess)
                    await print_files(omess,rdict)
                    torlog.info("Here are the files to be uploaded {}".format(rdict))
                else:
                    res = await rclone_driver(path,rmess, omess)
                    if res is None:
                        await msg.reply("<b>UPLOAD TO DRIVE FAILED CHECK LOGS</b>",parse_mode="html")
            
            try:
                if os.path.isdir(path):
                    shutil.rmtree(path)
                else:
                    os.remove(path)
            except:pass
    
    return None

async def pause_all(msg):
    await QBittorrentWrap.pause_all(msg)

async def resume_all(msg):
    await QBittorrentWrap.resume_all(msg)

async def purge_all(msg):
    await QBittorrentWrap.delete_all(msg)

async def get_status(msg,all=False):
    smsg = await QBittorrentWrap.get_status(msg,all)
    await msg.reply(smsg,parse_mode="html")

async def handle_zips(path, is_zip, rmess):
    # refetch rmess
    rmess = await rmess.client.get_messages(rmess.chat_id,ids=rmess.id)
    if is_zip:
        try:
            await rmess.edit(rmess.text+"\n Starting to Zip the contents. Please wait.")
            zip_path = await add_to_zip(path, get_val("TG_UP_LIMIT"))
            
            if zip_path is None:
                await rmess.edit(rmess.text+"\n Zip failed. Falback to normal")
                return False
            
            if os.path.isdir(path):
                shutil.rmtree(path)
            if os.path.isfile(path):
                os.remove(path)
            await rmess.edit(rmess.text+"\n Zipping Done now uploading.")
            return zip_path
        except:
            await rmess.edit(rmess.text+"\n Zip failed. Falback to normal")
            return False
    else:
        return path

async def handle_ext_zip(path, rmess, omess):
    # refetch rmess
    rmess = await rmess.client.get_messages(rmess.chat_id,ids=rmess.id)
    password = rmess.client.dl_passwords.get(omess.id)
    if password is not None:
        password = password[1]
    start = time.time()
    await rmess.edit(f"{rmess.text} Trying to Extract the archive with password <code>{password}</code>.", parse_mode="html")
    wrong_pwd = False

    while True:
        if not wrong_pwd:
            ext_path = await extract_archive(path,password=password)
        else:
            if (time.time() - start) > 1200:
                await rmess.edit(f"{rmess.text} Extract failed as no correct password was provided uploading as it is.")
                return False

            temppass = rmess.client.dl_passwords.get(omess.id)
            if temppass is not None:
                temppass = temppass[1]
            if temppass == password:
                await aio.sleep(10)
                continue
            else:
                password = temppass
                wrong_pwd = False
                continue
        
        if "Wrong Password" in ext_path:
            mess = f"<a href='tg://user?id={omess.sender_id}'>RE-ENTER PASSWORD</a>\nThe passowrd <code>{password}</code> you provided is a wrong password.You have {((time.time()-start)/60)-20} Mins to reply else un extracted zip will be uploaded.\n Use <code>/setpass {omess.id} password-here</code>"
            await omess.reply(mess, parse_mode="html")
            wrong_pwd = True
        elif ext_path is False:
            return False
        elif ext_path is None:
            # None is to descibe fetal but the upload will fail 
            # itself further nothing to handle here
            return False
        else:
            return ext_path
    



async def print_files(e,files,thash=None):
    msg = f"<a href='tg://user?id={e.sender_id}'>Done</a>\n#uploads\n"
    if len(files) == 0:
        return
    
    chat_id = e.chat_id
    msg_li = []
    for i in files.keys():
        link = f'https://t.me/c/{str(chat_id)[4:]}/{files[i]}'
        if len(msg + f'🚩 <a href="{link}">{i}</a>\n') > 4000:
            msg_li.append(msg)
            msg = f'🚩 <a href="{link}">{i}</a>\n'
        else:
            msg += f'🚩 <a href="{link}">{i}</a>\n'

    for i in msg_li:
        await e.reply(i,parse_mode="html")
        await aio.sleep(1)
        
    await e.reply(msg,parse_mode="html")

    try:
        if thash is not None:
            from .store_info_hash import store_driver # pylint: disable=import-error
            await store_driver(e, files, thash) 
    except:
        pass

    if len(files) < 2:
        return

    ids = list()
    for i in files.keys():
        ids.append(files[i])
    
    msgs = await e.client.get_messages(e.chat_id,ids=ids)
    for i in msgs:
        index = None
        for j in range(0,len(msgs)):
            index = j
            if ids[j] == i.id:
                break
        nextt,prev = "",""
        chat_id = str(e.chat_id)[4:]
        buttons = []
        if index == 0:
            nextt = f'https://t.me/c/{chat_id}/{ids[index+1]}'
            buttons.append(
                types.KeyboardButtonUrl("Next", nextt)
            )
            nextt = f'<a href="{nextt}">Next</a>\n'
        elif index == len(msgs)-1:
            prev = f'https://t.me/c/{chat_id}/{ids[index-1]}'
            buttons.append(
                types.KeyboardButtonUrl("Prev", prev)
            )
            prev = f'<a href="{prev}">Prev</a>\n'
        else:
            nextt = f'https://t.me/c/{chat_id}/{ids[index+1]}'
            buttons.append(
                types.KeyboardButtonUrl("Next", nextt)
            )
            nextt = f'<a href="{nextt}">Next</a>\n'
            
            prev = f'https://t.me/c/{chat_id}/{ids[index-1]}'
            buttons.append(
                types.KeyboardButtonUrl("Prev", prev)
            )
            prev = f'<a href="{prev}">Prev</a>\n'

        try:
            #await i.edit("{} {} {}".format(prev,i.text,nextt),parse_mode="html")
            await i.edit(buttons=buttons)
        except:pass
        await aio.sleep(1)
    


async def get_transfer():
    client = await QBittorrentWrap.get_client()
    data = client.transfer_info()
    print(data)
    return data

async def cancel_torrent(hashid, is_aria = False):
    if not is_aria:
        await QBittorrentWrap.deregister_torrent(hashid)
    else:
        await ariatools.remove_dl(hashid)



import os,logging,time,traceback,shutil
import aiohttp
from ..core.getVars import get_val
from ..core import thumb_manage # i guess i will dodge this one ;) as i am importing the vids helper anyways
from . import vids_helpers,zip7_utils
from .progress_for_telethon import progress
from hachoir.parser import createParser
from hachoir.metadata import extractMetadata
from telethon.errors import VideoContentTypeInvalidError
from ..core.database_handle import tkupload
from .. import user_db
from telethon.tl.types import KeyboardButtonCallback,DocumentAttributeVideo,DocumentAttributeAudio
from telethon.utils import get_attributes
from .Ftele import upload_file




env_email = get_val("EMAIL_KEY")
env_api_key = get_val("API_KEY")

torlog = logging.getLogger(__name__)


async def mixdrop_driver(path,message,from_uid,files_dict,job_id=0,force_edit=False,updb=None,from_in=False,thumb_path=None, user_msg=None):
    # creting here so connections are kept low
    if updb is None:
        # Central object is not used its Acknowledged 
        updb = tkupload()

    #logging.info("Uploading Now:- {}".format(path))

    if os.path.isdir(path):
        logging.info("Uplaoding the directory:- {}".format(path))

        directory_contents = os.listdir(path)
        directory_contents.sort()
        try:
            # maybe way to refresh?!
            message = await message.client.get_messages(message.chat_id,ids=[message.id])
            message = message[0]
        except:pass

        message = await message.edit("{}\nFound {} files for this download".format(message.text,len(directory_contents)))
        
        if not from_in:
            updb.register_upload(message.chat_id,message.id)
            if user_msg is None:
                sup_mes = await message.get_reply_message()
            else:
                sup_mes = user_msg
            
            data = "upcancel {} {} {}".format(message.chat_id,message.id,sup_mes.sender_id)
            buts = [KeyboardButtonCallback("Cancel upload.",data.encode("UTF-8"))]
            message = await message.edit(buttons=buts)


        for file in directory_contents:
            if updb.get_cancel_status(message.chat_id,message.id):
                continue

            await mixdrop_driver(
                os.path.join(path,file),
                message,
                from_uid,
                files_dict,
                job_id,
                force_edit,
                updb,
                from_in=True,
                thumb_path=thumb_path,
                user_msg=user_msg
            )
        
        if not from_in:
            if updb.get_cancel_status(message.chat_id,message.id):
                await message.edit("{} - Cancled By user.".format(message.text),buttons=None)
            else:
                await message.edit(buttons=None)
            updb.deregister_upload(message.chat_id,message.id)

    else:
        logging.info("Uplaoding the file:- {}".format(path))
        if os.path.getsize(path) > get_val("TG_UP_LIMIT"):
            # the splitted file will be considered as a single upload ;)
            
            
            metadata = extractMetadata(createParser(path))
            
            if metadata is not None:
                # handle none for unknown
                metadata = metadata.exportDictionary()
                try:
                    mime = metadata.get("Common").get("MIME type")
                except:
                    mime = metadata.get("Metadata").get("MIME type")

                ftype = mime.split("/")[0]
                ftype = ftype.lower().strip()
            else:
                ftype = "unknown"
            
            if ftype == "video":    
                todel = await message.reply("FILE LAGRE THEN THRESHOLD SPLITTING NOW.Processing.....\n```Using Algo FFMPEG SPLIT```") 
                split_dir = await vids_helpers.split_file(path,get_val("TG_UP_LIMIT"))
            else:
                todel = await message.reply("FILE LAGRE THEN THRESHOLD SPLITTING NOW.Processing.....\n```Using Algo PARTED ZIP SPLIT```") 
                split_dir = await zip7_utils.split_in_zip(path,get_val("TG_UP_LIMIT"))
            
            dircon = os.listdir(split_dir)
            dircon.sort()

            if not from_in:
                updb.register_upload(message.chat_id,message.id)
                if user_msg is None:
                    sup_mes = await message.get_reply_message()
                else:
                    sup_mes = user_msg
                data = "upcancel {} {} {}".format(message.chat_id,message.id,sup_mes.sender_id)
                buts = [KeyboardButtonCallback("Cancel upload.",data.encode("UTF-8"))]
                await message.edit(buttons=buts)

            for file in dircon:
                if updb.get_cancel_status(message.chat_id,message.id):
                    continue
            
                await mixdrop_driver(
                    os.path.join(split_dir,file),
                    message,
                    from_uid,
                    files_dict,
                    job_id,
                    force_edit,
                    updb=updb,
                    from_in=True,
                    thumb_path=thumb_path,
                    user_msg=user_msg
                )
            
            try:
                shutil.rmtree(split_dir)
                os.remove(path)
            except:pass
            
            if not from_in:
                if updb.get_cancel_status(message.chat_id,message.id):
                    await message.edit("{} - Cancled By user.".format(message.text),buttons=None)
                else:
                    await message.edit(buttons=None)
                updb.deregister_upload(message.chat_id,message.id)
            # spliting file logic blah blah
        else:
            if not from_in:
                updb.register_upload(message.chat_id,message.id)
                if user_msg is None:
                    sup_mes = await message.get_reply_message()
                else:
                    sup_mes = user_msg
                
                data = "upcancel {} {} {}".format(message.chat_id,message.id,sup_mes.sender_id)
                buts = [KeyboardButtonCallback("Cancel upload.",data.encode("UTF-8"))]
                await message.edit(buttons=buts)
            #print(updb)
            if not os.path.exists(path):
                sentmsg = None
            else:
                sentmsg = await mixFileup(
                    path,
                    message,
                    force_edit,
                    updb,
                    thumb_path,
                    user_msg
                )

            if not from_in:
                if updb.get_cancel_status(message.chat_id,message.id):
                    await message.edit("{} - Cancled By user.".format(message.text),buttons=None)
                else:
                    await message.edit(buttons=None)
                updb.deregister_upload(message.chat_id,message.id)

            if sentmsg is not None:
                files_dict[os.path.basename(path)] = sentmsg.id

    return files_dict


async def mixFileup(path,message,force_edit,database=None,thumb_path=None,user_msg=None):
    queue = message.client.queue
    if database is not None:
        if database.get_cancel_status(message.chat_id,message.id):
            # add os remove here
            return None
    if not os.path.exists(path):
        return None
    #todo improve this uploading ✔️
    file_name = os.path.basename(path)
    file_size = os.path.getsize(path)
    metadata = extractMetadata(createParser(path))
    ometa = metadata    
    if metadata is not None:
        # handle none for unknown
        metadata = metadata.exportDictionary()
        try:
            mime = metadata.get("Common").get("MIME type")
        except:
            mime = metadata.get("Metadata").get("MIME type")

        ftype = mime.split("/")[0]
        ftype = ftype.lower().strip()
    else:
        ftype = "unknown"
    if not force_edit:
        if user_msg is None:
            sup_mes = await message.get_reply_message()
        else:
            sup_mes = user_msg
        
        data = "upcancel {} {} {}".format(message.chat_id,message.id,sup_mes.sender_id)
        buts = [KeyboardButtonCallback("Cancel upload.",data.encode("UTF-8"))]
        msg = await message.reply("Uploading {}".format(file_name),buttons=buts)
    else:
        msg = message
        #print(metadata)
    uploader_id = None
    if queue is not None:
        torlog.info(f"Waiting for the worker here for {file_name}")
        msg = await msg.edit(f"{msg.text} - Waiting for a uploaders to get free")
        uploader_id = await queue.get()
        torlog.info(f"Waiting over for the worker here for {file_name} aquired worker {uploader_id}")
    out_msg = None
    start_time = time.time()
    tout = get_val("EDIT_SLEEP_SECS")
    opath = path

    try:
        email = env_email
        api_key = env_api_key
        upload_url = "https://ul.mixdrop.co/api"
        async with aiohttp.ClientSession() as session:
            data = {
                'file': open(path,"rb"),
                'email': email,
                'key': api_key
            }
            response = await session.post(upload_url, data=data)
            link = await response.json()
#            dl_b = f"https://mixdrop.co/f/{link['result']['fileref']}"
            torlog.info("success")            
    except Exception as e:
        if str(e).find("cancel") != -1:
            torlog.info("cancled an upload lol")
            await msg.delete()
        else:
            torlog.info(traceback.format_exc())
    finally:
        if queue is not None:
            await queue.put(uploader_id)
            torlog.info(f"Freed uploader with id {uploader_id}")
    if out_msg is None:
        return None
    if out_msg.id != msg.id:
        await msg.delete()
    
    return out_msg






#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# (c) Shrimadhav U K

# the logging things
import logging

import math
import os
import time
from .Human_Format import human_readable_bytes,human_readable_timedelta
from ..core.getVars import get_val

torlog = logging.getLogger(__name__)

async def progress_for_pyrogram(
    current,
    total,
    ud_type,
    message,
    start,
    time_out,
    client,
    cancel_msg=None,
    updb=None,
    markup=None
):
    now = time.time()
    diff = now - start
    
    # too early to update the progress
    if diff < 1:
        return
    
    if round(diff % time_out) == 0 or current == total:
        if cancel_msg is not None:
            # dirty alt. was not able to find something to stop upload
            # todo inspect with "StopAsyncIteration"
            # IG Open stream will be Garbage Collected
            if updb.get_cancel_status(cancel_msg.chat.id,cancel_msg.message_id):
                print("Stopping transmission")
                client.stop_transmission()
    
        # if round(current / total * 100, 0) % 5 == 0:
        percentage = current * 100 / total
        elapsed_time = round(diff)
        speed = current / elapsed_time
        time_to_completion = round((total - current) / speed)
        estimated_total_time = elapsed_time + time_to_completion

        elapsed_time = time_formatter(elapsed_time)
        estimated_total_time = time_formatter(estimated_total_time)

        progress = "`[{0}{1}]` \nprogress: `{2}%`\n".format(
            ''.join([get_val("COMPLETED_STR") for _ in range(math.floor(percentage / 5))]),
            ''.join([get_val("REMAINING_STR") for _ in range(20 - math.floor(percentage / 5))]),
            round(percentage, 2))

        tmp = progress + "uploaded: `{0} of {1}`\nspeed: `{2}/s`\neta: `{3}`\n".format(
            humanbytes(current),
            humanbytes(total),
            humanbytes(speed),
            estimated_total_time if estimated_total_time != '' else "0 seconds"
        )
        try:
            if not message.photo:
                await message.edit_text(
                    text="`{}`\n `{}`".format(
                        ud_type,
                        tmp
                    ),
                    reply_markup=markup
                )
            else:
                await message.edit_caption(
                    caption="`{}`\n `{}`".format(
                        ud_type,
                        tmp
                    ),
                    reply_markup=markup
                )
        except:
            pass


def humanbytes(size):
    # https://stackoverflow.com/a/49361727/4723940
    # 2**10 = 1024
    if not size:
        return ""
    power = 2**10
    n = 0
    Dic_powerN = {0: ' ', 1: 'Ki', 2: 'Mi', 3: 'Gi', 4: 'Ti'}
    while size > power:
        size /= power
        n += 1
    return str(round(size, 2)) + " " + Dic_powerN[n] + 'B'


def time_formatter(seconds: int) -> str:
    result = ""
    v_m = 0
    remainder = seconds
    r_ange_s = {
        "days": (24 * 60 * 60),
        "hours": (60 * 60),
        "minutes": 60,
        "seconds": 1
    }
    for age in r_ange_s:
        divisor = r_ange_s[age]
        v_m, remainder = divmod(remainder, divisor)
        v_m = int(v_m)
        if v_m != 0:
            result += f" {v_m} {age} "
    return result

